import { Button } from '@material-ui/core'
import Dialog from '@material-ui/core/Dialog'
import EnedisIcon from 'assets/icons/ico/consent-outdated-enedis.svg'
import errorIcon from 'assets/icons/visu/data-nok.svg'
import successIcon from 'assets/icons/visu/data-ok.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import connectionWaitingText from 'constants/connectionWaitingText.json'
import firstConnectionWaitingText from 'constants/firstConnectionWaitingText.json'
import {
  ERROR_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { FluidType, KonnectorError } from 'enums'
import { shuffle } from 'lodash'
import { Account } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useAppSelector } from 'store/hooks'
import { getFluidName } from 'utils/utils'
import KonnectorModalFooter from './KonnectorModalFooter'
import './konnectorModal.scss'

interface KonnectorModalProps {
  open: boolean
  /** Only used for SGE when searching for user identity */
  isVerifyingIdentity: boolean
  state: string | null
  error: KonnectorError | null
  fluidType: FluidType
  handleCloseClick: (isSuccess?: boolean) => void
  handleAccountDeletion: () => Promise<void>
  account: Account | null
}

const KonnectorModal = ({
  open,
  isVerifyingIdentity,
  state,
  error,
  fluidType,
  handleCloseClick,
  handleAccountDeletion,
  account,
}: KonnectorModalProps) => {
  const { t } = useI18n()
  const fluidName = getFluidName(fluidType)
  const { fluidStatus } = useAppSelector(state => state.ecolyo.global)
  const isUpdating = fluidStatus[fluidType].connection.isUpdating
  const [index, setIndex] = useState<number>(0)
  /** Only used for enedis to see common errors */
  const [showCommonErrors, setShowCommonErrors] = useState(false)

  const shuffledWaitingTexts = useMemo(() => {
    if (fluidType) {
      return shuffle(connectionWaitingText)
    }
    return connectionWaitingText
  }, [fluidType])

  const firstConnectionWaitingTexts = firstConnectionWaitingText.concat(
    ...shuffledWaitingTexts
  )

  const getUpdatingText = useCallback(() => {
    return (
      <div className="kmodal-waiting-text text-18-italic">
        {shuffledWaitingTexts.map((text, idx) => (
          <div
            key={idx}
            className={classNames('waiting-text', {
              ['show']: idx === index % shuffledWaitingTexts.length,
            })}
          >
            <p>{text.first}</p>
            <p>{text.second}</p>
          </div>
        ))}
      </div>
    )
  }, [index, shuffledWaitingTexts])

  const getConnectionText = useCallback(() => {
    return (
      <div className="kmodal-waiting-text text-18-italic">
        {isVerifyingIdentity ? (
          <p className="text-18-white">{t('konnector_modal.logging_txt')}</p>
        ) : (
          firstConnectionWaitingTexts.map((text, idx) => (
            <div
              key={idx}
              className={classNames('waiting-text', {
                ['show']: idx === index % firstConnectionWaitingTexts.length,
              })}
            >
              <p>{text.first}</p>
              <p>{text.second}</p>
            </div>
          ))
        )}
      </div>
    )
  }, [firstConnectionWaitingTexts, index, isVerifyingIdentity, t])

  /** Returns connection success contents, depending on the fluid and update status */
  const connectionSuccessContent = () => (
    <div className="konnector-config">
      <StyledIcon icon={successIcon} size={48} />
      <div className="headerSuccess text-20-bold">
        {t(`konnector_modal.success_${isUpdating ? 'update_' : ''}txt`)}
      </div>
      <b>
        {t(
          `konnector_modal.success_data_${
            isUpdating ? 'update_' : ''
          }${fluidName}`
        )}
      </b>
      <div
        className="light"
        dangerouslySetInnerHTML={{
          __html: t(
            `konnector_modal.success_data_additional_${
              isUpdating ? 'update_' : ''
            }${fluidName}`
          ),
        }}
      />
    </div>
  )

  useEffect(() => {
    const interval = setInterval(() => {
      if (open && !state) {
        setIndex(prev => prev + 1)
      }
    }, 8000)
    return () => {
      clearInterval(interval)
    }
  }, [open, state])

  if (fluidType === FluidType.GAS && error === KonnectorError.CHALLENGE_ASKED) {
    return null
  }

  return (
    <Dialog
      open={open}
      disableEscapeKeyDown
      onClose={(event, reason): void => {
        event &&
          reason !== 'backdropClick' &&
          handleCloseClick(state === SUCCESS_EVENT)
      }}
      aria-labelledby="accessibility-title"
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <div id="accessibility-title">
        {t('konnector_modal.accessibility.window_title')}
      </div>
      <div className="kmodal-content">
        {/* NEITHER ERROR NOR SUCCESS => LOADING */}
        {!state ? (
          <>
            <Loader fluidType={fluidType} />
            {!isVerifyingIdentity && (
              <div className="kmodal-content-text text-16-normal">
                <div className="text-16-bold">
                  {t(
                    `konnector_modal.loading_data${isUpdating ? '_update' : ''}`
                  )}
                </div>
              </div>
            )}
            {isUpdating && getUpdatingText()}
            {!isUpdating && getConnectionText()}
          </>
        ) : (
          <>
            {/* ERROR OR SUCCESS */}
            <div className="kmodal-info">
              {state === ERROR_EVENT && (
                <>
                  {error === KonnectorError.LOGIN_FAILED && (
                    // LOGIN FAILED FOR ENEDIS AND EGL
                    <div className="konnector-config">
                      <StyledIcon icon={errorIcon} size={48} />
                      <div className="headerError text-20-bold">
                        {t('konnector_modal.error_txt')}
                      </div>
                      <div>
                        {t(
                          `konnector_modal.error_credentials_${
                            isUpdating ? 'update_' : ''
                          }${fluidName}`
                        )}
                      </div>
                      {fluidType === FluidType.ELECTRICITY && !isUpdating && (
                        <div className="elec-fail">
                          {t(
                            `konnector_modal.error_credentials_${fluidName}_2`
                          )}
                        </div>
                      )}
                      {/* Show common errors for enedis */}
                      {fluidType === FluidType.ELECTRICITY && (
                        <>
                          {!showCommonErrors ? (
                            <Button
                              className="btnText"
                              onClick={() => setShowCommonErrors(true)}
                            >
                              {t('konnector_modal.show_common_error')}
                            </Button>
                          ) : (
                            <div
                              className="commonErrorsList"
                              dangerouslySetInnerHTML={{
                                __html: t(
                                  'konnector_modal.show_common_error_list'
                                ),
                              }}
                            />
                          )}
                        </>
                      )}
                    </div>
                  )}
                  {error === KonnectorError.TERMS_VERSION_MISMATCH &&
                    isUpdating &&
                    fluidType === FluidType.ELECTRICITY && (
                      // MISMATCH UPDATE ERROR ENEDIS
                      <div className="headerError konnector-config mismatch">
                        <StyledIcon icon={EnedisIcon} width={120} height={80} />
                        <div className="title text-20-bold">
                          {t('konnector_modal.mismatch.title')}
                        </div>
                        <div className="text-15-white">
                          {t('konnector_modal.mismatch.text1')}
                        </div>
                        <div className="text-15-white">
                          {t('konnector_modal.mismatch.text2')}
                        </div>
                        <div className="info text-15-normal">
                          {t('konnector_modal.mismatch.text3')}
                        </div>
                      </div>
                    )}
                  {error ===
                    KonnectorError.USER_ACTION_NEEDED_ACCOUNT_REMOVED &&
                    fluidType === FluidType.GAS && (
                      <div className="konnector-config">
                        <StyledIcon icon={errorIcon} size={48} />
                        <div className="headerError text-20-bold">
                          {t('konnector_modal.error_txt')}
                        </div>
                        <div className="title text-16-bold">
                          {t('konnector_modal.error_consent_form_gas_title')}
                        </div>
                        <div className="light text-14-regular">
                          {t('konnector_modal.error_consent_form_gas_report')}
                        </div>
                      </div>
                    )}

                  {error !== KonnectorError.LOGIN_FAILED &&
                    error !== KonnectorError.TERMS_VERSION_MISMATCH &&
                    error !==
                      KonnectorError.USER_ACTION_NEEDED_ACCOUNT_REMOVED && (
                      // DEFAULT CASE
                      <div className="konnector-config">
                        <StyledIcon icon={errorIcon} size={48} />
                        <div className="headerError text-20-bold">
                          {t('konnector_modal.error_txt')}
                        </div>
                        <div className="text-15-bold">
                          {t(
                            `konnector_modal.error_data_${
                              isUpdating ? 'update_' : ''
                            }${fluidName}`
                          )}
                        </div>
                        <div className="text-14-regular">
                          {t('konnector_modal.error_data_2')}
                        </div>
                      </div>
                    )}
                </>
              )}
              {state === SUCCESS_EVENT && connectionSuccessContent()}
              <KonnectorModalFooter
                state={state}
                error={error}
                handleCloseClick={handleCloseClick}
                handleAccountDeletion={handleAccountDeletion}
                account={account}
                isUpdating={isUpdating}
              />
            </div>
          </>
        )}
      </div>
    </Dialog>
  )
}

export default KonnectorModal
